from supabase import create_client, Client
from supabase.lib.client_options import ClientOptions
from .config import Settings

settings = Settings()

def get_supabase(jwt: str = None) -> Client:
    if jwt and jwt != "" and len(jwt.split(".")) == 3:
        return create_client(
            settings.SUPABASE_URL,
            settings.SUPABASE_KEY,
            options=ClientOptions(
                headers={"Authorization": f"Bearer {jwt}"}
            )
        )
    else:
        return create_client(settings.SUPABASE_URL, settings.SUPABASE_KEY)



from functools import wraps
from fastapi import HTTPException
import asyncio

def error_handler(f):
    @wraps(f)
    async def wrapper(*args, **kwargs):
        try:
            # Use 'await' for async functions
            if asyncio.iscoroutinefunction(f):
                return await f(*args, **kwargs)
            else:
                return f(*args, **kwargs)
        except HTTPException as http_exc:
            print(f"{f.__name__} - HTTP Exception: {http_exc.detail}")
            raise
        except Exception as exc:
            print(f"{f.__name__} - Unhandled Exception: {exc}")
            raise HTTPException(status_code=400, detail="An error occurred")
    return wrapper

# Usage example
# @app.get("/items/{item_id}")
# @error_handler  # Apply the decorator directly
# async def read_item(item_id: int):
#     # Your endpoint logic here
#     pass



from fyodorov_llm_agents.agents.agent_model import Agent as AgentModel
import yaml

def parse_yaml(yaml_str: str) -> dict:
    if not yaml_str:
        raise ValueError('YAML string is required')
    print(f"Parsing YAML: {yaml_str}")
    yaml_str = yaml_str.strip()
    yaml_dict = yaml.safe_load(yaml_str)
    print(f"yaml_dict: {yaml_dict}")
    if 'version' not in yaml_dict:
        raise ValueError('fyodorov config version is required')
    providers = []
    for provider in yaml_dict['providers']:
        print(f"Parsing provider: {provider}")
        print(f"body: {yaml_dict['providers'][provider]}")
        provider = ProviderModel(**yaml_dict['providers'][provider])
        provider.validate()
        providers.append(provider)
    print(f"providers: {providers}")
    models = []
    print(f"Parsing agents YAML in format: {yaml_dict['version']}")
    agents = []
    for agent in yaml_dict['agents']:
        print(f"Parsing agent: {agent}")
        print(f"body: {yaml_dict['agents'][agent]}")
        agent = AgentModel(**yaml_dict['agents'][agent])
        agent.validate()
        agents.append(agent)
    print(f"agents: {agents}")
    return {
        'agents': agents
    }



from typing import Union
from datetime import datetime, timedelta
from fyodorov_utils.config.supabase import get_supabase
from fyodorov_llm_agents.tools.tool import Tool as ToolModel
from .base import Base

class Tool(Base):
    def create_from_plugin(access_token: str, plugin: dict) -> ToolModel:
        print("in func creating tool from plugin")
        tool = ToolModel.from_plugin(plugin)
        print('created tool from plugin', tool)
        return Tool.create_in_db(access_token, tool)

    @staticmethod    
    def create_in_db(access_token: str, tool: ToolModel) -> str:
        try:
            supabase = get_supabase(access_token)
            result = supabase.table('tools').insert(tool.to_dict()).execute()
            tool_id = result.data[0]['id']
            return tool_id
        except Exception as e:
            print('Error creating tool', str(e))
            raise e

    @staticmethod
    def update_in_db(access_token: str, id: str, tool: dict) -> dict:
        if not id:
            raise ValueError('Tool ID is required')
        try:
            supabase = get_supabase(access_token)
            result = supabase.table('tools').update(tool).eq('id', id).execute()
            return result.data[0]
        except Exception as e:
            print('An error occurred while updating tool:', id, str(e))
            raise

    @staticmethod
    def delete_in_db(access_token: str, id: str) -> bool:
        if not id:
            raise ValueError('Tool ID is required')
        try:
            supabase = get_supabase(access_token)
            result = supabase.table('tools').delete().eq('id', id).execute()
            return True
        except Exception as e:
            print('Error deleting tool', str(e))
            raise e

    @staticmethod
    def get_in_db(access_token: str, id: str) -> ToolModel:
        if not id:
            raise ValueError('Tool ID is required')
        try:
            supabase = get_supabase(access_token)
            result = supabase.table('tools').select('*').eq('id', id).limit(1).execute()
            tool_dict = result.data[0]
            print('got tool from db', tool_dict)
            tool = ToolModel(**tool_dict)
            return tool
        except Exception as e:
            print('Error fetching tool', str(e))
            raise e

    @staticmethod
    def get_by_name_and_user_id(access_token: str, name: str, user_id: str) -> ToolModel:
        if not name:
            raise ValueError('Tool name is required')
        try:
            supabase = get_supabase(access_token)
            result = supabase.table('tools').select('*').eq('name_for_ai', name).eq('user_id', user_id).limit(1).execute()
            if not result.data: # If no tools found for this user check for public tools with same name
                result = supabase.table('tools').select('*').eq('name_for_ai', name).eq('public', True).limit(1).execute()
            if not result.data:
                print(f"No tool found with name {name} and user_id {user_id}")
                return None
            tool_dict = result.data[0]
            tool_dict['name'] = tool_dict['name_for_ai']
            tool_dict['description'] = tool_dict['description_for_ai']
            print('got tool from db', tool_dict)
            tool = ToolModel(**tool_dict)
            return tool
        except Exception as e:
            print('Error fetching tool', str(e))
            raise e

    @staticmethod
    def get_all_in_db(access_token: str, limit: int = 10, created_at_lt: datetime = datetime.now()) -> [dict]:
        try:
            supabase = get_supabase(access_token)
            result = supabase.from_('tools') \
                .select("*") \
                .limit(limit) \
                .lt('created_at', created_at_lt) \
                .order('created_at', desc=True) \
                .execute()
            tools = result.data
            return tools
        except Exception as e:
            print('Error fetching tools', str(e))
            raise e


from typing import Union
from datetime import datetime, timedelta
from fyodorov_utils.config.supabase import get_supabase

class Model():
    id: str = None

    def to_dict(self):
        return {c.name: str(getattr(self, c.name)) for c in self.__table__.columns}

    def from_dict(self, data):
        for field in self.__table__.columns:
            if field.name in data:
                setattr(self, field.name, data[field.name])

class Base(Model):
    table: str = 'ERROR: table not set'

    @staticmethod    
    def create_in_db(access_token: str) -> str:
        try:
            supabase = get_supabase(access_token)
            result = supabase.table(self.table).insert(self.to_dict()).execute()
            self.id = result.data[0]['id']
            return self.id
        except Exception as e:
            print('Error creating object in table', self.table, str(e))
            raise e

    @staticmethod
    def update_in_db(access_token: str, id: str) -> dict:
        if not id:
            raise ValueError('ID is required to update in table', self.table)
        try:
            self.id = id
            supabase = get_supabase(access_token)
            result = supabase.table(self.table).update(self.to_dict()).eq('id', self.id).execute()
            return result.data[0]
        except Exception as e:
            print('An error occurred while updating table', self.table, id, str(e))
            raise

    @staticmethod
    def delete_in_db(access_token: str, id: str) -> bool:
        if not id:
            raise ValueError('ID is required to delete from table', self.table)
        try:
            self.id = id
            supabase = get_supabase(access_token)
            result = supabase.table(self.table).delete().eq('id', self.id).execute()
            return True
        except Exception as e:
            print('Error deleting from table', self.table, str(e))
            raise e

    @staticmethod
    def get_in_db(access_token: str, id: str) -> Model:
        if not id:
            raise ValueError('ID is required to get from table', self.table)
        try:
            self.id = id
            supabase = get_supabase(access_token)
            result = supabase.table(self.table).select('*').eq('id', self.id).limit(1).execute()
            self.from_dict(result.data[0])
            return self.base_object
        except Exception as e:
            print('Error fetching tool', str(e))
            raise e

    @staticmethod
    def get_all_in_db(access_token: str, limit: int = 10, created_at_lt: datetime = datetime.now()) -> [dict]:
        try:
            supabase = get_supabase(access_token)
            result = supabase.from_(self.table) \
                .select("*") \
                .limit(limit) \
                .lt('created_at', created_at_lt) \
                .order('created_at', desc=True) \
                .execute()
            tools = result.data
            return tools
        except Exception as e:
            print('Error fetching multiple objects from table', self.table, str(e))
            raise e